# -*- coding: utf-8 -*-
# Description:
# Ancor Client library
#
# Authors:
# Benoît Allard <benoit.allard@greenbone.net>
#
# Copyright:
# Copyright (C) 2014 Greenbone Networks GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

""" OSP Ancor client class. """

import logging
import requests
import ospd.misc

logger = logging.getLogger(__name__)


class AncorClient(object):
    """ Ancor client class. """

    def __init__(self, server, key, session):
        logger.info('initializing Ancor Client')
        logger.debug('Server is: %s', server)
        self._server = server
        # Make sure the key is all on one line
        self._key = key.replace('\n', '')
        self._session = session
        self._session_id = None

    def _has_session(self):
        return self._session_id is not None

    def _endpoint(self, name):
        return '/'.join(['https:/', self._server, 'AncorWebService', name])

    def login(self):
        if self._has_session():
            return True
        url = self._endpoint('login')
        headers = {'Authorization': 'SAML ' + self._key, }

        result = self._session.get(url, headers=headers)
        if result.status_code != 200:
            logger.error('Error while logging-in: %d (%s)', result.status_code,
                         result.text)
            return False
        self._session_id = result.text
        return True

    def logout(self):
        if not self._has_session():
            return False
        url = self._endpoint('logout')
        headers = {'Authorization': 'Bearer ' + self._session_id}

        result = self._session.get(url, headers=headers)
        if result.status_code != 200:
            logger.error('Error while logging-out: %d (%s)', result.status_code,
                         result.text)
            return False
        self._session_id = None
        return True

    def get_results(self, target):
        """ Get the results from Ancor. """

        headers = {'Authorization': 'SAML ' + self._key, }
        if ospd.misc.target_to_ipv4(target):
            url = self._endpoint('gethostresultswithip')
            payload = {'ip': target}
        else:
            url = self._endpoint('gethostresultswithhostname')
            payload = {'hostname': target}
        try:
            response = self._session.post(url, data=payload, headers=headers)
        except requests.ConnectionError as message:
            logger.error("HTTP post failed: %s", message)
            return None, str(message)
        if response.status_code != 200:
            logger.error('Error while searching: %d (%s)', response.status_code,
                         response.text)
            return None, response.text
        try:
            results = response.json()
            return results, None
        except ValueError as message:
            logger.error('Error while decoding response: %s', message)
            return None, str(message)
