# -*- coding: utf-8 -*-
# Description:
# OSP Ancor Daemon wrapper
#
# Authors:
# Benoît Allard <benoit.allard@greenbone.net>
#
# Copyright:
# Copyright (C) 2014-2015 Greenbone Networks GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

""" OSP ancor class. """

import logging

import ospd.misc
from ospd.ospd import OSPDaemon, OSPDError
import requests

from ospd_ancor.client import AncorClient
from ospd_ancor import __version__

logger = logging.getLogger(__name__)

OSPD_ANCOR_DESC = """
This scanner accesses the SecPod Ancor platform which offers
vulnerability information collected by the SecPod Saner End-Point-Security
solution.

Running this OSP scanner will not initiate a vulnerability scan
but rather query a database for the latest available vulnerability
information for the given target systems. Which also means that
the scan takes only a short time to finish.

For more information on the SecPod Saner product line see:
    http://www.secpod.com/


Technical functionality:

Each "concern" objects retrieved from Ancor that does have a reference to
a CVE will represent a single scan result.

Additionally, the "asset" information for each host will be retrieved as
host details.
"""

ANCOR_PARAMS = {
    'address': {
        'type': 'string',
        'name': 'Ancor server Address',
        'default': '',
        'description': 'The address of the Ancor server to connect to',
    },
    'token': {
        'type': 'file',
        'name': 'Ancor authentitcation key',
        'default': '',
        'description': 'The ancor authentication token as provided by SecPod',
    },
}


def cve_metadata_severity(cve_metadata, default='10.0'):
    """ Get severity value from CVE metadata. """
    logger.debug('CVE Metadata searching ...')
    for metadata in cve_metadata.split('||'):
        if '=' not in metadata:
            continue
        name, value = metadata.split('=', 1)
        if name != 'Score':
            continue
        logger.info("==> %s: %s", name, value)
        return value
    return default

def process_asset(asset):
    """ Process result for an asset. """
    asset_name = asset['name']
    logger.info('Processing result for %s', asset_name)
    concerns = asset.get('concerns', {})
    concern = concerns.get('concern', [])
    if len(concern) == 0:
        logger.debug('No concern')
        yield dict(name=asset_name)
    for _concern in concern:
        severity = ''
        if 'cve-metadata' in _concern:
            severity = cve_metadata_severity(_concern['cve-metadata'])
        else:
            logger.debug('No CVE metadata found')
            # We can't extract severity from CCE metadata
        desc = _concern['desc']

        yield dict(name=asset_name, value=desc, severity=severity)


def process_sysinfo(sysinfo):
    """ Yield (name, value) pairs for the host properties """
    logger.debug('Analysing sysinfo: %s', sysinfo)
    yield 'cpuinfo', sysinfo['architecture']
    # gsa does not support multiple interfaces for one host (yet).
    interface = sysinfo['interfaces']['interface'][0]
    yield 'NIC', interface['interface_name']
    yield 'MAC', interface['mac_address']
    yield 'NIC_IPS', interface['ip_address']
    # Remaining information has no gsa equivalent (Well, cpe:/o...)
    # yield 'os_name', sysinfo['os_name']
    # yield 'os_version', sysinfo['os_version']
    # yield 'primary_host_name', sysinfo['primary_host_name']


class OSPDAncor(OSPDaemon):
    """ Class for ospd-ancor daemon. """

    def __init__(self, *args, **kwargs):
        OSPDaemon.__init__(self, *args, **kwargs)
        self.server_version = __version__
        self.scanner_info['name'] = 'Ancor'
        self.scanner_info['version'] = 'unknown version'
        self.scanner_info['description'] = OSPD_ANCOR_DESC
        for name, param in ANCOR_PARAMS.items():
            self.add_scanner_param(name, param)

    def check(self):
        # We can't do anything here as we don't know the server address / token
        return True

    def process_scan_params(self, params):
        """ params is directly from the XML """
        if not params.get('address'):
            raise OSPDError('Invalid address value', 'start_scan')
        if not params.get('token'):
            raise OSPDError('Invalid token value', 'start_scan')
        return params

    def exec_scan(self, scan_id, target):
        options = self.get_scan_options(scan_id)
        assert 'address' in options
        assert 'token' in options

        session = requests.Session()
        # XXX: This should go away !
        session.verify = False
        session.headers.update(
            {'User-agent': ' '.join([self.get_daemon_name(),
                                     self.get_daemon_version()])})
        client = AncorClient(options['address'], options['token'], session)

        logger.info('Requesting results for target: %s', target)
        response, errmsg = client.get_results(target)
        if response is None:
            self.add_scan_error(scan_id, host=target,
                                value=errmsg)
        else:
            self.process_response(scan_id, target, response)

    def process_response(self, scan_id, target, response):
        """ Process the response from ancor server. """

        saner_results = response.get('sanerResults', [None])
        for saner_result in saner_results:
            if saner_result is None:
                self.add_scan_error(scan_id, host=target, value='No results')
                break
            results = saner_result.get('results', {})
            assets = results.get('assets', {})
            asset = assets.get('asset', [])
            for _asset in asset:
                for result in process_asset(_asset):
                    if 'severity' in result:
                        self.add_scan_alarm(scan_id, host=target, **result)
                    else:
                        self.add_scan_log(scan_id, host=target, **result)

            for name, value in process_sysinfo(results['sysinfo']):
                self.add_scan_host_detail(
                    scan_id, host=target, name=name, value=value)


def main():
    """ OSP ancor main function. """
    ospd.misc.main('OSPD - ancor wrapper', OSPDAncor)
